/*
 * @file Client_Module_Im.c
 * Module Interface for Module Client_Module_Im
 */

#include "Client_Module_Im.h"
#include <stdio.h>

////////////////////////////////////////////////////////////
// User functions.
static void testAddition(Client_Module_Im__context *context)
{
   ECOA__log log;
   ECOA__return_status status;

   ECOA__int32 value1 = 5;
   ECOA__int32 value2 = 10;
   ECOA__int32 result = 0;

   log.current_size = sprintf((char *) &log.data, "requesting addition of %d and %d", value1, value2);
   Client_Module_Im_container__log_info(context, log);

   status = Client_Module_Im_container__add__request_sync(context, value1, value2, &result);

   log.current_size = sprintf((char *) &log.data, "result of addition of %d and %d = %d", value1, value2, result);
   Client_Module_Im_container__log_info(context, log);
}

static void testSubtraction(Client_Module_Im__context *context)
{
   ECOA__log log;
   ECOA__return_status status;
   Client_Module_Im_container__BasicMathServiceAvailable_handle availableHandle;

   ECOA__int32 value1 = 50;
   ECOA__int32 value2 = 10;
   ECOA__int32 result = 0;

   // The subtract operation should check if the service is functionally available, as the server
   // behaviour is defined to not respond if a request is received when functionally unavailable.
   // This could lead to the client module being blocked indefinitely if no timeout is specified!
   status = Client_Module_Im_container__BasicMathServiceAvailable__get_read_access(context, &availableHandle);

   if (status == ECOA__return_status_OK)
   {
      if (*(availableHandle.data) == ECOA__TRUE)
      {
         log.current_size = sprintf((char *) &log.data, "requesting subtraction of %d minus %d", value1, value2);
         Client_Module_Im_container__log_info(context, log);

         status = Client_Module_Im_container__subtract__request_sync(context, value1, value2, &result);

         log.current_size = sprintf((char *) &log.data, "result of subtraction of %d minus %d = %d", value1, value2, result);
         Client_Module_Im_container__log_info(context, log);
      }
      else
      {
         log.current_size = sprintf((char *) &log.data, "cannot perform subtraction as service unavailable");
         Client_Module_Im_container__log_info(context, log);
      }

      status = Client_Module_Im_container__BasicMathServiceAvailable__release_read_access(context, &availableHandle);
   }
}

static void testMultiplication(Client_Module_Im__context *context)
{
   ECOA__log log;
   ECOA__return_status status;
   Client_Module_Im_container__BasicMathServiceAvailable_handle availableHandle;

   ECOA__int32 value1 = 7;
   ECOA__int32 value2 = 8;
   ECOA__int32 result = 0;

   // When the server is declaring its service as unavailable, then it will return a default value for the multiplication.
   // When the server stops periodically publishing its availability then assume the service is not available
   status = Client_Module_Im_container__BasicMathServiceAvailable__get_read_access(context, &availableHandle);

   if (status == ECOA__return_status_OK)
   {
      if(availableHandle.stamp != context->user.previousStamp)
      {
         context->user.previousStamp = availableHandle.stamp;
         log.current_size = sprintf((char *) &log.data, "requesting multiplication of %d by %d", value1, value2);
         Client_Module_Im_container__log_info(context, log);

         status = Client_Module_Im_container__multiply__request_sync(context, value1, value2, &result);

         log.current_size = sprintf((char *) &log.data, "result of multiplication of %d by %d = %d", value1, value2, result);
         Client_Module_Im_container__log_info(context, log);

      }
      else
      {
         log.current_size = sprintf((char *) &log.data, "server availability is stale - not requesting multiplication");
         Client_Module_Im_container__log_info(context, log);
      }

      status = Client_Module_Im_container__BasicMathServiceAvailable__release_read_access(context, &availableHandle);
   }
}

static void testDivision(Client_Module_Im__context *context)
{
   ECOA__log log;
   ECOA__return_status status;

   ECOA__int32 value1 = 1000;
   ECOA__int32 value2 = 20;
   ECOA__int32 result = 0;
   BasicMath__Divide_Status_Type divideStatus;

   log.current_size = sprintf((char *) &log.data, "requesting division of %d by %d", value1, value2);
   Client_Module_Im_container__log_info(context, log);

   status = Client_Module_Im_container__divide__request_sync(context, value1, value2, &result, &divideStatus);

   if (divideStatus == BasicMath__Divide_Status_Type_OK)
   {
      log.current_size = sprintf((char *) &log.data, "result of division of %d by %d = %d", value1, value2, result);
      Client_Module_Im_container__log_info(context, log);
   }
   else
   {
      log.current_size = sprintf((char *) &log.data, "Failed to divide - status = %d", divideStatus);
      Client_Module_Im_container__log_info(context, log);
   }
}

static void testPower(Client_Module_Im__context *context)
{
   ECOA__log log;
   ECOA__return_status status;

   ECOA__int32 base = 3;
   ECOA__int32 exponent = 4;
   ECOA__int32 result = 0;

   log.current_size = sprintf((char *) &log.data, "requesting %d raised to the power %d", base, exponent);
   Client_Module_Im_container__log_info(context, log);

   status = Client_Module_Im_container__power__request_sync(context, base, exponent, &result);

   log.current_size = sprintf((char *) &log.data, "result of %d raised to the power %d = %d", base, exponent, result);
   Client_Module_Im_container__log_info(context, log);
}

static void testSquareRoot(Client_Module_Im__context *context)
{
   ECOA__log log;
   ECOA__return_status status;

   ECOA__int32 value1 = 25;
   ECOA__int32 result = 0;

   log.current_size = sprintf((char *) &log.data, "requesting square root of %d", value1);
   Client_Module_Im_container__log_info(context, log);

   status = Client_Module_Im_container__squareRoot__request_sync(context, value1, &result);

   log.current_size = sprintf((char *) &log.data, "result of square root of %d = %d", value1, result);
   Client_Module_Im_container__log_info(context, log);
}
////////////////////////////////////////////////////////////

void Client_Module_Im__INITIALIZE__received(Client_Module_Im__context *context)
{
   /* User Code Here */
	context->user.previousStamp = 0;
}

void Client_Module_Im__START__received(Client_Module_Im__context *context)
{
   /* User Code Here */
}

void Client_Module_Im__STOP__received(Client_Module_Im__context *context)
{
   /* User Code Here */
}

void Client_Module_Im__SHUTDOWN__received(Client_Module_Im__context *context)
{
   /* User Code Here */
}

void Client_Module_Im__tick__received(Client_Module_Im__context *context)
{
   ECOA__return_status status;

   // Basic math should only be used if the service has been set as functionally available...
   // However, we can use any except subtract as the other service operations are designed to send a response regardless.
   testAddition(context);
   testSubtraction(context);
   testMultiplication(context);
   testDivision(context);

   // The exponential math can be used anytime...
   testPower(context);
   testSquareRoot(context);

   ECOA__log log;
   log.current_size = sprintf((char *) &log.data, "--------------------------------------------------");
   Client_Module_Im_container__log_info(context, log);

}
