/*
 * @file BasicMath_Im.c
 * Module Interface for Module BasicMath_Im
 */

#include "BasicMath_Im.h"
#include <stdio.h>

static void Publish_Functional_Service_Availability(BasicMath_Im__context* context)
{
	   ECOA__return_status status;
	   BasicMath_Im_container__available_handle availableHandle;

	   status = BasicMath_Im_container__available__get_write_access(context, &availableHandle);

	   if (status == ECOA__return_status_OK || status == ECOA__return_status_DATA_NOT_INITIALIZED)
	   {
	      *(availableHandle.data) = context->user.basicMathServiceAvailable;
	   }

	   status = BasicMath_Im_container__available__publish_write_access(context, &availableHandle);
}

void BasicMath_Im__INITIALIZE__received(BasicMath_Im__context *context)
{
   /* Initially set the functional service unavailable */
   context->user.publish = ECOA__TRUE;
   context->user.basicMathServiceAvailable = ECOA__FALSE;
   Publish_Functional_Service_Availability(context);
   context->user.availCount = 0;
}

void BasicMath_Im__START__received(BasicMath_Im__context *context)
{
}

void BasicMath_Im__STOP__received(BasicMath_Im__context *context)
{
   /* User Code Here */
}

void BasicMath_Im__SHUTDOWN__received(BasicMath_Im__context *context)
{
   /* User Code Here */
}

void BasicMath_Im__add__request_received
   (BasicMath_Im__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 value1,
    const ECOA__int32 value2)
{
   // The behaviour of the add operation is:
   // Undefined if the service is not "functionally" available. The module does not check the state before responding!

   ECOA__return_status status;

   ECOA__int32 result = value1 + value2;
   status = BasicMath_Im_container__add__response_send(context, ID, result);

   if(context->user.availCount <= 2)
   {
      context->user.availCount++;
   }

}

void BasicMath_Im__subtract__request_received
   (BasicMath_Im__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 value1,
    const ECOA__int32 value2)
{
   // The behaviour of the subtract operation is:
   // Check if the service is functionally available.
   // If it is available, send a response.
   // If it is unavailable, do not send a response.

   ECOA__return_status status;

   if (context->user.basicMathServiceAvailable)
   {
      ECOA__int32 result = value1 - value2;
      status = BasicMath_Im_container__subtract__response_send(context, ID, result);
   }
}

void BasicMath_Im__multiply__request_received
   (BasicMath_Im__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 value1,
    const ECOA__int32 value2)
{
   // The behaviour of the multiply operation is:
   // Check if the service is functionally available.
   // If it is available, send a response.
   // If it is unavailable, send a response, but with a default value.

   ECOA__return_status status;
   ECOA__int32 result = 0;

   if (context->user.basicMathServiceAvailable)
   {
      result = value1 * value2;
      status = BasicMath_Im_container__multiply__response_send(context, ID, result);
   }
   else
   {
      status = BasicMath_Im_container__multiply__response_send(context, ID, result);
   }
}

void BasicMath_Im__divide__request_received
   (BasicMath_Im__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 value1,
    const ECOA__int32 value2)
{
   // The behaviour of the divide operation is:
   // Check if the service is functionally available.
   // If it is available, send a response.
   // If it is unavailable, send a response, but with an "Unavailable" status and default value.

   ECOA__return_status status;
   ECOA__int32 result = 0;
   BasicMath__Divide_Status_Type divideStatus = BasicMath__Divide_Status_Type_OK;

   if (context->user.basicMathServiceAvailable)
   {
      ECOA__int32 result = value1 / value2;
      status = BasicMath_Im_container__divide__response_send(context, ID, result, divideStatus);
   }
   else
   {
      // Return not available.
      divideStatus = BasicMath__Divide_Status_Type_Unavailable;
      status = BasicMath_Im_container__divide__response_send(context, ID, result, divideStatus);
   }

}

void BasicMath_Im__publish__received(BasicMath_Im__context *context)
{
   ECOA__log log;

   /* User Code Here */
   if(context->user.availCount < 11)
   {
      context->user.availCount++;
   }

   switch(context->user.availCount)
   {
      case 6:
         log.current_size = sprintf((char *) &log.data, "**** Server now setting service available");
		 BasicMath_Im_container__log_info(context, log);

		 context->user.basicMathServiceAvailable = ECOA__TRUE;

	     break;
      case 10:
         log.current_size = sprintf((char *) &log.data, "**** Server stopping publish");
		 BasicMath_Im_container__log_info(context, log);

		 context->user.publish = ECOA__FALSE;

	     break;
   }

   if(context->user.publish)
   {
		 Publish_Functional_Service_Availability(context);
   }
}
